from fiscalsim_us.model_api import *


class la_total_additions(Variable):
    """
    Additions to Louisiana adjusted income
    Lines 2A-D on Form IT-540 Schedule E. These additions include
    * Interest and dividend income from other states and their political subdivisions
    * Recapture of START contributions
    * Recapture of START K12 contributions
    * Add back of pass-through entity
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana total additions to AGI"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA



from fiscalsim_us.model_api import *


class la_income_tax_before_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana income tax before refundable credits"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )
    defined_for = StateCode.LA

    def formula(tax_unit, period, parameters):
        itax_before_credits = tax_unit("la_income_tax_before_credits", period)
        nonrefundable_credits = tax_unit("la_nonrefundable_credits", period)
        return max_(0, itax_before_credits - nonrefundable_credits)



from fiscalsim_us.model_api import *


class la_other_nonrefundable_p3(Variable):
    """
    Louisiana nonrefundable priority 3 credits not yet implimented as separate variables
    Form IT-540 line 20
    These include the following credits with codes:
    202 – Organ Donation
    208 – Previously Unemployed
    221 – Owner of Accessible and Barrier-free Home –
    224 – New Jobs Credit
    228 – Eligible Re-entrants –
    236 – Apprenticeship (2007)
    251 – Motion Picture Investment
    252 – Research and Development
    253 – Historic Structures
    254 – Digital Interactive Media
    257 – Capital Company
    258 – LA Community Development Financial Institution (LCDFI)
    259 – New Markets
    261 – Motion Picture Infrastructure
    262 – Angel Investor
    300 – Biomed/University Research
    305 – Tax Equalization
    310 – Manufacturing Establishment
    412 – Refund by Utilities
    424 – Donation to School Tuition Organization
    454 – QMC Music Job Creation Credit
    457 – Neighborhood Assistance
    458 – Research and Development
    459 – Ports of Louisiana Import Export Cargo
    460 – LA Import
    461 – LA Work Opportunity
    462 – Youth Jobs
    463 – Apprenticeship (2022)
    464 – Donation to Qualified Foster Care Charitable Organization
    500 – Inventory Tax Credit Carried Forward and ITEP
    502 – Ad Valorem Natural Gas Credit Carried Forward
    504 – Atchafalaya Trace
    506 – Cane River Heritage
    508 – Ports of Louisiana Investor
    510 – Enterprise Zone
    550 – Recycling Credit
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana nonrefundable priority 3 income tax credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )



from fiscalsim_us.model_api import *


class la_total_subtractions(Variable):
    """
    Subtractions from louisiana adjusted income
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana total subtractions from AGI"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    adds = "gov.states.la.tax.income.agi.subtractions.sources"



from fiscalsim_us.model_api import *


class la_refundable_priority_2_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana refundable priority 2 income tax credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )
    adds = "gov.states.la.tax.income.credits.refundable_p2.sources"



from fiscalsim_us.model_api import *


class la_eitc(Variable):
    """
    Louisiana earned income tax credit
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana EITC"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )
    defined_for = StateCode.LA

    def formula(tax_unit, period, parameters):
        federal_eitc = tax_unit("earned_income_tax_credit", period)
        p = parameters(period).gov.states.la.tax.income.credits.refundable_p2

        return federal_eitc * p.eitc_mult



from fiscalsim_us.model_api import *


class la_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana refundable income tax credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )

    def formula(tax_unit, period, parameters):
        priority_2_refund = tax_unit(
            "la_refundable_priority_2_credits", period
        )

        # line 18 - floor at 0 since the priority 3 credits are nonrefundable
        tax_afer_p_2_credits = max_(
            0,
            tax_unit("la_income_tax_before_refundable_credits", period)
            - priority_2_refund,
        )

        priority_3_nonrefund_amount = tax_unit(
            "la_nonrefundable_priority_3_credits", period
        )

        # either refund the rest of the tax liability (leftover from p2)
        # or refund the total amount of p3 credits, whichever is less
        priority_3_nonrefund_add = min_(
            tax_afer_p_2_credits, priority_3_nonrefund_amount
        )

        priority_4_refund = tax_unit(
            "la_refundable_priority_4_credits", period
        )

        return priority_2_refund + priority_3_nonrefund_add + priority_4_refund



from fiscalsim_us.model_api import *


class sc_senior_exemption(Variable):
    value_type = float
    entity = TaxUnit
    label = "South Carolina senior exemption"
    unit = USD
    definition_period = YEAR
    reference = "https://dor.sc.gov/forms-site/Forms/SC1040_2022.pdf"
    defined_for = StateCode.SC

    def formula(tax_unit, period, parameters):
        # First get their filing status.
        filing_status = tax_unit("filing_status", period)

        # Then get the SC senior exemptions part of the parameter tree
        p = parameters(period).gov.states.sc.tax.income.exemptions.senior

        # Get the individual filer's age.
        age_head = tax_unit("age_head", period)

        # Determine if head of household (filer) is eligible.
        head_eligible = (age_head >= p.age_threshold).astype(int)

        # Get the spouse age, if applicable.
        age_spouse = tax_unit("age_spouse", period)

        # Determine whether spouse is eligible (>= age 65).
        joint = filing_status == filing_status.possible_values.JOINT
        spouse_eligible = ((age_spouse >= p.age_threshold) * joint).astype(int)

        # Get SC retirement income deduction and military retirement income deduction

        head_deductions = add(
            tax_unit,
            period,
            [
                "sc_retirement_income_deduction_head",
                "sc_military_retirement_income_deduction_head",
            ],
        )
        spouse_deductions = add(
            tax_unit,
            period,
            [
                "sc_retirement_income_deduction_spouse",
                "sc_military_retirement_income_deduction_spouse",
            ],
        )

        # Calculate senior exemption. The exemption can not be less than 0. Add head and spouse together.
        # Per the legal code, this applies separately:
        # "(2) In the case of married taxpayers who file a joint federal income tax return, the reduction required by item (1) applies to each individual separately"
        head_exemption = max_(head_eligible * p.amount - head_deductions, 0)
        spouse_exemption = max_(
            spouse_eligible * p.spouse_amount - spouse_deductions, 0
        )
        return head_exemption + spouse_exemption



from fiscalsim_us.model_api import *


class la_taxable_income(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana taxable income"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )
    defined_for = StateCode.LA

    def formula(tax_unit, period, parameters):
        ADDS = ["adjusted_gross_income", "la_total_additions"]
        SUBTRACTS = ["la_total_subtractions", "la_exemptions"]

        la_agi = add(tax_unit, period, ADDS) - add(tax_unit, period, SUBTRACTS)

        # LA allows tax units to subtract from income any portion of federal
        # medical expense deductions that is more than the standard deduction
        filing_status = tax_unit("filing_status", period)
        itemizing = tax_unit("tax_unit_itemizes", period)
        std_deduct = parameters(period).gov.irs.deductions.standard.amount[
            filing_status
        ]
        medical_deduct = tax_unit("medical_expense_deduction", period)

        excess_fed_deduct = max_(0, medical_deduct - std_deduct)

        #        return max_(0, la_agi - excess_fed_deduct)
        return where(itemizing, la_agi - excess_fed_deduct, la_agi)



from fiscalsim_us.model_api import *


class la_other_refundable_p2(Variable):
    """
    Other Louisiana refundable priority 2 credits listed
    on Schedule F for IT-540. These credits include the
    following with codes:
    * 52F - Ad Valorem Offshore Vessels
    * 54F – Telephone Company Property
    * 54F – Telephone Company Property
    * 58F – Milk Producers
    * 59F – Technology Commercialization
    * 60F – Historic Residential
    * 62F – Musical and Theatrical Production
    * 65F – School Readiness Child Care Provider
    * 66F – School Readiness Child Care Directors and Staff
    * 67F – School Readiness Business-Supported Child Care
    * 68F – School Readiness Fees and Grants to Resource and
    Referral Agencies
    * 70F – Retention and Modernization
    * 73F – Digital Interactive Media & Software
    * 76F – Stillborn Child
    * 77F – Funeral and Burial Expense for a Pregnancy-related Death
    * 80F – Other Refundable Credit
    """

    unit = USD
    value_type = float
    entity = TaxUnit
    label = "Louisiana other refundable priority 2 income tax credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )



from fiscalsim_us.model_api import *


class la_state_teacher_retirement_benefits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana state teacher retirement benefits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA



from fiscalsim_us.model_api import *


class vt_personal_exemptions(Variable):
    value_type = float
    entity = TaxUnit
    label = "Vermont personal exemptions"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.VT
    reference = (
        "https://tax.vermont.gov/sites/tax/files/documents/IN-111-2022.pdf"
    )

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.vt.tax.income.exemption
        # First, Need to determine the tax unit filing status, if they are filing jointly
        filing_status = tax_unit("filing_status", period)
        is_joint = filing_status == filing_status.possible_values.JOINT
        # Then, determine whether either the head or the spouse of the tax unit is claimable as a dependent in another unit.
        claimable_elsewhere_head = tax_unit("dsi", period)
        claimable_elsewhere_spouse = tax_unit("dsi_spouse", period)
        # If claimable elsewhere, it is not eligible for vt personal exemption (line 5a, line 5b)
        eligible_head = (~claimable_elsewhere_head).astype(int)
        eligible_spouse = (~claimable_elsewhere_spouse).astype(int)
        eligible_count = eligible_head + (eligible_spouse * is_joint)
        # Last, add number of other dependents claimed on federal Form 1040.(line 5c)
        dependents = tax_unit("tax_unit_count_dependents", period)
        total_exemption_count = eligible_count + dependents
        return total_exemption_count * p.personal



from fiscalsim_us.model_api import *


class sc_gross_earned_income(Variable):
    value_type = float
    entity = Person
    label = "South Carolina gross earned income"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.SC

    adds = "gov.states.sc.tax.income.credits.two_wage_earner.earned_income"



from fiscalsim_us.model_api import *


class az_agi(Variable):
    value_type = float
    entity = TaxUnit
    label = "Arizona adjusted gross income"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.AZ



from fiscalsim_us.model_api import *


class military_retirement_pay(Variable):
    value_type = float
    entity = Person
    label = "ME military retirement pay subtractions"
    unit = USD
    definition_period = YEAR
    documentation = "The benefits received under a United States military retirement plan, including survivor benefits, are fully exempt from Maine income tax. See 2022 - Worksheet for Pension Income Deduction below."
    reference = "https://www.maine.gov/revenue/sites/maine.gov.revenue/files/inline-files/22_1040me_sched_1s_ff.pdf"



from fiscalsim_us.model_api import *


class la_nonrefundable_credits(Variable):
    """
    Louisiana nonrefundable priority 1 credits
    Line 11 on Form IT-540. These credits include
    the following with codes:
    * 100 - Premium tax credit
    * 120 - Bone Marrow credit
    * 150 - Qualified Playgrounds
    * 155 - Debt Issuance
    * 199 - Other
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana nonrefundable credits"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )
    defined_for = StateCode.LA



from fiscalsim_us.model_api import *


class la_exemptions(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana exemptions amount"
    unit = USD
    definition_period = YEAR
    reference = ()
    defined_for = StateCode.LA

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.la.tax.income.exemptions
        filing_status = tax_unit("filing_status", period)
        statuses = filing_status.possible_values
        single_separate = (filing_status == statuses.SINGLE) | (
            filing_status == statuses.SEPARATE
        )

        dependents = tax_unit("tax_unit_dependents", period)
        aged_blind = tax_unit("aged_blind_count", period)

        personal_multiplier = where(single_separate, 1, 2)
        n_extra = dependents + aged_blind

        return (
            p.personal_amount * personal_multiplier
            + p.additional_amount * n_extra
        )



from fiscalsim_us.model_api import *


class la_other_subtractions(Variable):
    """
    Other subtractions from louisiana taxable income.
    These include the following exemptions from income with
    the associated codes:
    05E – Other Retirement Benefits
    06E – Annual Retirement Income Exemption for Taxpayers 65
    Years of Age or Older
    08E – Native American Income
    09E – START Savings Program Contribution
    10E – Military Pay Exclusion
    11E – Road Home
    13E – Recreation Volunteer
    14E – Volunteer Firefighter
    16E – Voluntary Retrofit Residential Structure
    17E – Elementary and Secondary School Tuition
    18E – Educational Expenses for Home-Schooled Children
    19E – Educational Expenses for a Quality Public Education
    20E – Capital Gain from Sale of Louisiana Business
    21E – Employment of Certain Qualified Disabled Individuals
    22E – S Bank Shareholder Income Exclusion
    23E – Entity Level Taxes Paid to Other States
    24E – Pass–Through Entity Exclusion
    25E – IRC 280C Expense
    27E – COVID-19 Relief Benefits
    28E – START K12 Savings Program Contributions
    29E – Digital Nomad
    30E – Certain Adoptions
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana other subtractions from AGI"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA



from fiscalsim_us.model_api import *


class la_nonrefundable_childcare(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana refundable childcare credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.la.tax.income.credits.nonrefundable_p3.nonrefundable_child_care
        agi = tax_unit("adjusted_gross_income", period)
        fed_cdcc = tax_unit("cdcc", period)

        # for agi under income floor can use the refundable credit
        nonrefundable = agi > p.income_floor
        fed_cdcc = fed_cdcc * nonrefundable

        income_mult = p.income_mult.calc(agi)
        credit = fed_cdcc * income_mult

        # for agi over the threshold the credit is the minimum of
        # fed_cdcc * income_mult and the limit for above the threshold
        return where(
            agi > p.income_threshold,
            min_(credit, p.above_threshold_max),
            credit,
        )



from fiscalsim_us.model_api import *


class la_federal_retirement_benefits(Variable):
    """Amount of retirement benefits recieved from a
    Federal Retirement System
    """

    value_type = float
    entity = TaxUnit
    label = "Louisiana state teacher retirement benefits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    # If this gets implimented in a federal level variable
    # delete this one and change in parameter subtractions.sources.yaml



from fiscalsim_us.model_api import *


class nd_subtractions(Variable):
    value_type = float
    entity = TaxUnit
    label = "ND subtractions from federal taxable income"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.tax.nd.gov/sites/www/files/documents/forms/form-nd-1-2021.pdf"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/2021-individual-income-tax-booklet.pdf"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/form-nd-1-2022.pdf"
        "https://www.tax.nd.gov/sites/www/files/documents/forms/2022-individual-income-tax-booklet.pdf"
    )
    defined_for = StateCode.ND
    adds = "gov.states.nd.tax.income.taxable_income.subtractions.sources"



from fiscalsim_us.model_api import *


class nj_main_income_tax(Variable):
    value_type = float
    entity = TaxUnit
    label = "New Jersey income tax"
    unit = USD
    definition_period = YEAR
    reference = "https://law.justia.com/codes/new-jersey/2022/title-54a/section-54a-2-1/"
    defined_for = StateCode.NJ

    def formula(tax_unit, period, parameters):
        taxable_income = tax_unit("nj_taxable_income", period)
        filing_status = tax_unit("filing_status", period)
        status = filing_status.possible_values

        # Get main nj tax parameter tree.
        p = parameters(period).gov.states.nj.tax.income.main

        return select(
            [
                filing_status == status.SINGLE,
                filing_status == status.JOINT,
                filing_status == status.HEAD_OF_HOUSEHOLD,
                filing_status == status.WIDOW,
                filing_status == status.SEPARATE,
            ],
            [
                p.single.calc(taxable_income),
                p.joint.calc(taxable_income),
                p.head_of_household.calc(taxable_income),
                p.widow.calc(taxable_income),
                p.separate.calc(taxable_income),
            ],
        )



from fiscalsim_us.model_api import *


class la_refundable_childcare(Variable):
    value_type = float
    entity = TaxUnit
    label = "Louisiana refundable childcare credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.LA
    reference = (
        "https://revenue.louisiana.gov/TaxForms/IT540WEB(2022)%20F%20D2.pdf",
        "https://revenue.louisiana.gov/TaxForms/IT540iWEB(2022)D1.pdf",
    )

    def formula(tax_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.la.tax.income.credits.refundable_p2.refundable_child_care
        agi = tax_unit("adjusted_gross_income", period)
        childcare_expenses = tax_unit("tax_unit_childcare_expenses", period)

        # check for eligible children
        person = tax_unit.members
        dependent = person("is_tax_unit_dependent", period)
        age = person("age", period)
        child = age < p.child_age_limit
        eligible = dependent & child
        eligible_children = tax_unit.sum(eligible)

        max_expense = p.expense_limit.calc(eligible_children)
        expense = min_(childcare_expenses, max_expense)

        # line 6 of worksheet
        tax_unit_min_income = tax_unit("min_head_spouse_earned", period)
        credit = min_(expense, tax_unit_min_income)

        income_mult = p.income_mult.calc(agi)
        mult = p.mult

        return credit * income_mult * mult



from fiscalsim_us.model_api import *


class ky_standard_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "Kentucky standard deduction"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.KY

    def formula(tax_unit, period, parameters):
        return parameters(period).gov.states.ky.tax.income.deductions.standard



from fiscalsim_us.model_api import *


class ky_income_tax_before_non_refundable_credits(Variable):
    value_type = float
    entity = TaxUnit
    label = "Kentucky income tax before non-refundable credits"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.KY

    def formula(tax_unit, period, paramters):
        income = tax_unit("ky_taxable_income", period)
        rate = paramters(period).gov.states.ky.tax.income.rate
        return income * rate
